#pragma once

#include <a8/timer_attacher.h>

namespace cs
{
    class CMJoin;
}

class Room;
class RoomMgr : public a8::Singleton<RoomMgr>
{
 public:
    enum { HID = HID_RoomMgr };

 private:
    RoomMgr() {};
    friend class a8::Singleton<RoomMgr>;

 public:
    void Init();
    void UnInit();
    void Update(int delta_time);

    void _CMJoin(f8::MsgHdr& hdr, const cs::CMJoin& msg);
    void ActiveRoom(long long room_uuid);
    int RoomNum();
    int OverRoomNum();
    Room* GetRoomByUuid(long long uuid);
    void AddOverRoom(long long room_uuid);
    bool IsGM(const std::string& account_id);

 private:
    void InstallReportStateTimer();
    Room* GetRoomByIdx(int room_idx);
    Room* GetJoinableRoom(const cs::CMJoin& msg,
                          const RoomType_e self_room_type,
                          int game_times,
                          int creator_register_time);
    void ReportServerState(int instance_id, const std::string& host, int port);
    void FreeOverRoom(long long room_uuid);
    bool IsLimitJoin();

    int AllocRoomIdx();
    Room* CreateRoom(const cs::CMJoin& msg,
                     RoomType_e room_type,
                     int game_times,
                     int creator_register_time);
    void JoinErrorHandle(const cs::CMJoin& msg, int error_code, int socket_handle);

 private:
    int current_room_idx_ = 0;

    std::map<long long, Room*> inactive_room_hash_;
    std::map<long long, Room*> room_hash_;
    std::map<int, Room*> room_idx_hash_;
    std::map<long long, Room*> over_room_hash_;
    a8::TimerAttacher reportstate_timer_attacher_;
    std::map<std::string, int> gm_hash_;
};
